@@ -31,7 +31,8 @@ function add_feed($feedname, $function) {
 		$wp_rewrite->feeds[] = $feedname;
 	}
 	$hook = 'do_feed_' . $feedname;
-	remove_action($hook, $function, 10, 1);
+	// Remove default function hook
+	remove_action($hook, $hook, 10, 1);
 	add_action($hook, $function, 10, 1);
 	return $hook;
 }
@@ -59,6 +60,18 @@ function add_rewrite_endpoint($name, $places) {
 	$wp_rewrite->add_endpoint($name, $places);
 }
 
+/**
+  * _wp_filter_taxonomy_base() - filter the URL base for taxonomies, to remove any manually prepended /index.php/
+  * @param string $base the taxonomy base that we're going to filter
+  * @return string
+  * @author Mark Jaquith 
+  */
+function _wp_filter_taxonomy_base( $base ) {
+	if ( !empty( $base ) )
+		$base = preg_replace( '|^/index\.php/|', '/', $base );
+	return $base;
+}
+
 // examine a url (supposedly from this blog) and try to
 // determine the post ID it represents.
 function url_to_postid($url) {
@@ -793,12 +806,12 @@ class WP_Rewrite {
 		$robots_rewrite = array('robots.txt$' => $this->index . '?robots=1');
 
 		//Default Feed rules - These are require to allow for the direct access files to work with permalink structure starting with %category%
-		$default_feeds = array(	'.*/wp-atom.php$'	=>	$this->index .'?feed=atom',
-								'.*/wp-rdf.php$'	=>	$this->index .'?feed=rdf',
-								'.*/wp-rss.php$'	=>	$this->index .'?feed=rss',
-								'.*/wp-rss2.php$'	=>	$this->index .'?feed=rss2',
-								'.*/wp-feed.php$'	=>	$this->index .'?feed=feed',
-								'.*/wp-commentsrss2.php$'	=>	$this->index . '?feed=rss2&withcomments=1');
+		$default_feeds = array(	'.*wp-atom.php$'	=>	$this->index .'?feed=atom',
+								'.*wp-rdf.php$'	=>	$this->index .'?feed=rdf',
+								'.*wp-rss.php$'	=>	$this->index .'?feed=rss',
+								'.*wp-rss2.php$'	=>	$this->index .'?feed=rss2',
+								'.*wp-feed.php$'	=>	$this->index .'?feed=feed',
+								'.*wp-commentsrss2.php$'	=>	$this->index . '?feed=rss2&withcomments=1');
 
 		// Post
 		$post_rewrite = $this->generate_rewrite_rules($this->permalink_structure, EP_PERMALINK);
@@ -980,8 +993,8 @@ class WP_Rewrite {
 		if ($this->using_index_permalinks()) {
 			$this->root = $this->index . '/';
 		}
-		$this->category_base = get_option( 'category_base' );
-		$this->tag_base = get_option( 'tag_base' );
+		$this->category_base = ( ( $this->using_index_permalinks() ) ? '/' . $this->index : '' ) . get_option( 'category_base' );
+		$this->tag_base = ( ( $this->using_index_permalinks() ) ? '/' . $this->index : '' ) . get_option( 'tag_base' );
 		unset($this->category_structure);
 		unset($this->author_structure);
 		unset($this->date_structure);
