@@ -1,5 +1,11 @@
 <?php
-/*
+/**
+ * PHP-Gettext External Library: StreamReader classes
+ *
+ * @package External
+ * @subpackage PHP-gettext
+ *
+ * @internal
    Copyright (c) 2003, 2005 Danilo Segan <danilo@kvota.net>.
 
    This file is part of PHP-gettext.
@@ -18,7 +24,7 @@
    along with PHP-gettext; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-*/
+ */
 
 
 // Simple class to wrap file streams, string streams, etc.
@@ -153,7 +159,8 @@ class CachedFileReader extends StringReader {
 	return false;
       }
       $this->_str = fread($fd, $length);
-      fclose($fd);
+	  $this->_pos = 0;
+	  fclose($fd);
 
     } else {
       $this->error = 2; // File doesn't exist
@@ -163,4 +170,4 @@ class CachedFileReader extends StringReader {
 }
 
 
-?>
\ No newline at end of file
+?>
